@@ -6,7 +6,7 @@ import asyncio
 from dataclasses import dataclass
 from importlib import metadata
 import socket
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, Literal
 
 from aiohttp import ClientError, ClientResponseError, ClientSession
 from aiohttp.hdrs import METH_GET, METH_POST
@@ -14,11 +14,20 @@ from yarl import URL
 
 from .exceptions import OverseerrConnectionError
 from .models import (
+    MediaType,
+    MovieDetails,
     NotificationType,
     RequestCount,
+    RequestFilterStatus,
+    RequestResponse,
+    RequestSortStatus,
+    RequestWithMedia,
     Result,
     SearchResult,
     Status,
+    TVDetails,
+    WatchlistEntry,
+    WatchlistResponse,
     WebhookNotificationConfig,
 )
 
@@ -84,7 +93,7 @@ class OverseerrClient:
             msg = "Error occurred while communicating with the service"
             raise OverseerrConnectionError(msg) from exception
 
-        if response.status != 200:
+        if response.status >= 400:
             content_type = response.headers.get("Content-Type", "")
             text = await response.text()
             msg = "Unexpected response from Overseerr"
@@ -115,6 +124,69 @@ class OverseerrClient:
         response = await self._request(METH_GET, "settings/notifications/webhook")
         return WebhookNotificationConfig.from_json(response)
 
+    async def get_requests(
+        self,
+        status: RequestFilterStatus | None = None,
+        sort: RequestSortStatus | None = None,
+        requested_by: int | None = None,
+    ) -> list[RequestWithMedia]:
+        """Get requests from Overseerr."""
+        params: dict[str, Any] = {}
+        if status:
+            params["filter"] = status
+        if sort:
+            params["sort"] = sort
+        if requested_by:
+            params["requestedBy"] = requested_by
+        response = await self._request(METH_GET, "request", params=params)
+        return RequestResponse.from_json(response).results
+
+    async def create_request(
+        self,
+        media_type: MediaType,
+        tmdb_id: int,
+        seasons: list[int] | Literal["all"] | None = None,
+    ) -> RequestWithMedia:
+        """Create a request in Overseerr."""
+        data = {"mediaType": media_type, "mediaId": tmdb_id}
+        if seasons:
+            data["seasons"] = seasons
+        response = await self._request(METH_POST, "request", data=data)
+        return RequestWithMedia.from_json(response)
+
+    async def get_movie_details(self, identifier: int) -> MovieDetails:
+        """Get movie details from Overseerr."""
+        response = await self._request(METH_GET, f"movie/{identifier}")
+        return MovieDetails.from_json(response)
+
+    async def get_tv_details(self, identifier: int) -> TVDetails:
+        """Get tv details from Overseerr."""
+        response = await self._request(METH_GET, f"tv/{identifier}")
+        return TVDetails.from_json(response)
+
+    async def get_watchlist(self) -> list[WatchlistEntry]:
+        """Get watchlist from Overseerr."""
+        response = await self._request(METH_GET, "discover/watchlist")
+        return WatchlistResponse.from_json(response).results
+
+    async def test_webhook_notification_config(
+        self, webhook_url: str, json_payload: str
+    ) -> bool:
+        """Test webhook notification config with Overseerr."""
+        try:
+            await self._request(
+                METH_POST,
+                "settings/notifications/webhook/test",
+                data={
+                    "enabled": True,
+                    "types": NotificationType.REQUEST_PENDING_APPROVAL,
+                    "options": {"webhookUrl": webhook_url, "jsonPayload": json_payload},
+                },
+            )
+        except OverseerrConnectionError:
+            return False
+        return True
+
     async def set_webhook_notification_config(
         self,
         *,
